@@ -25,10 +25,9 @@ use strict;
 use warnings;
 
 use WWW::GoodData::Agent;
-use JSON;
+use JSON::XS;
 use URI;
 
-our $VERSION = '1.11';
 our $root = new URI ('https://secure.gooddata.com/gdc');
 
 =head1 METHODS
@@ -66,15 +65,17 @@ sub new
 	bless $self, $class;
 	$self->{agent} ||= new WWW::GoodData::Agent ($root);
 	$self->{retries} ||= 3600;
+	$self->{agent}->{error_callback} = \&error_callback;
 	return $self;
 }
 
 # API hierarchy traversal Cache
 our %links;
-sub get_canonical_links
+sub get_links
 {
 	my $self = shift;
-	my $root = shift;
+
+	my $root = (ref $_[0] and ref $_[0] ne 'HASH') ? shift : new URI ($self->{agent}{root});
 	my @path = map { ref $_ ? $_ : { category => $_ } } @_;
 	my $link = shift @path;
 
@@ -103,19 +104,18 @@ sub get_canonical_links
 				# Metadata with interesting information outside "links"
 				if (exists $element->{$type}{links}{self}
 					and exists $element->{$type}{meta}) {
-					my $link = new URI ($element->{$type}{links}{self})->abs ($root);
 					push @{$links{$root}}, {
 						%{$element->{$type}{meta}},
 						category => $type,
 						structure => $structure,
-						link => $link,
+						link => $element->{$type}{links}{self},
 					};
-					$root = $link;
+					$root = $element->{$type}{links}{self};
 				}
 
 				# The links themselves
 				foreach my $category (keys %{$element->{$type}{links}}) {
-					my $link = new URI ($element->{$type}{links}{$category})->abs ($root);
+					my $link = $element->{$type}{links}{$category};
 					push @{$links{$root}}, {
 						structure => $structure,
 						category => $category,
@@ -130,8 +130,14 @@ sub get_canonical_links
 		}
 	}
 
-	# Canonicalize the links
-	$_->{link} = new URI ($_->{link})->abs ($root) foreach @{$links{$root}};
+	# Uploads are on different server, but the link is incorrect
+	foreach (@{$links{$root}}) {
+		$_->{link} eq '/uploads' or next;
+		my $diroot = new URI ($_->{link}, $root->scheme)->abs ($root);
+		$diroot->host =~ /([^\.]*)(.*)/
+			and $diroot->host ("$1-di$2");
+		$_->{link} = "$diroot";
+	};
 
 	my @matches = grep {
 		my $this_link = $_;
@@ -148,26 +154,10 @@ sub get_canonical_links
 
 	die 'Nonexistent component in path' unless @matches;
 	die 'Ambigious path' unless scalar @matches == 1;
+	my $new_root = new URI ($matches[0]->{link});
+	$new_root = $new_root->abs ($root);
 
-	# Traverse further
-	return $self->get_canonical_links ($matches[0]->{link}, @path);
-}
-
-# This is a 'normalized' version, for convenience and compatibility
-sub get_links
-{
-	my $self = shift;
-	my $root = (ref $_[0] and ref $_[0] ne 'HASH') ? shift : '';
-
-	# Canonicalize URIs
-	$root = new URI ($root)->abs ($self->{agent}{root});
-
-	# And decanonicalize, ommiting the scheme and authority part if possible
-	my @links = $self->get_canonical_links ($root, @_);
-	$_->{link} = $_->{link}->rel ($root)->authority
-		?  $_->{link} : new URI ($_->{link}->path) foreach @links;
-
-	return @links;
+	return $self->get_links ($new_root, @path);
 }
 
 =item B<links> PATH
@@ -179,7 +169,7 @@ PATH is an array of dictionaries, where each key-value pair
 matches properties of a link. If a plain string is specified,
 it is considered to be a match against B<category> property:
 
-  $gdc->links ('md', { 'category' => 'projects' });
+  $gdc->get_links ('md', { 'category' => 'projects' });
 
 The above call returns a list of all projects, with links to
 their metadata resources.
@@ -218,9 +208,11 @@ sub login
 	my $self = shift;
 	my ($login, $password) = @_;
 
+	# We should really get a link here instead of mangling
+	# service root location...
 	my $root = new URI ($self->{agent}{root});
-	my $staging = $self->get_uri ('uploads')->abs ($root);
-	my $netloc = $staging->host.':'.$staging->port;
+	my $netloc = $root->host.':'.$root->port;
+	$netloc =~ s/([^\.]*)/$1-di/;
 
 	$self->{agent}->credentials ($netloc,
 		'GoodData project data staging area', $login => $password);
@@ -248,9 +240,9 @@ sub logout
 	die 'Not logged in' unless defined $self->{login};
 
 	# Forget Basic authentication
-	my $root = new URI ($self->{agent}{root});
-	my $staging = $self->get_uri ('uploads')->abs ($root);
-	my $netloc = $staging->host.':'.$staging->port;
+	my $netloc = $root->host.':'.$root->port;
+	# Neither on the same address, not navigatable
+	$netloc =~ s/([^\.])/$1-di/;
 	$self->{agent}->credentials ($netloc,
 		'GoodData project data staging area', undef, undef);
 
@@ -262,34 +254,6 @@ sub logout
 	$self->{login} = undef;
 }
 
-=item B<change_passwd> OLD NEW
-
-Change user password given the old and new password.
-
-=cut
-
-sub change_passwd
-{
-	my $self = shift;
-	my $old_passwd = shift or die 'No old password given';
-	my $new_passwd = shift or die 'No new password given';
-
-	die 'Not logged in' unless defined $self->{login};
-
-	my $profile = $self->{agent}->get ($self->{login}{userLogin}{profile});
-	my $new_profile = {
-		'accountSetting' => {
-			'old_password' => $old_passwd,
-			'password' => $new_passwd,
-			'verifyPassword' => $new_passwd,
-			'firstName' => $profile->{accountSetting}->{firstName},
-			'lastName' => $profile->{accountSetting}->{lastName}
-		}
-	};
-
-	$self->{agent}->put ($self->{login}{userLogin}{profile}, $new_profile);
-}
-
 =item B<projects>
 
 Return array of links to project resources on metadata server.
@@ -323,17 +287,11 @@ sub delete_project
 	$self->{agent}->delete ($uri);
 }
 
-=item B<create_project> TITLE SUMMARY TEMPLATE DRIVER TOKEN
+=item B<create_project> TITLE SUMMARY
 
-Create a project given its title and optionally summary, project template,
-db engine driver and authorization token
+Create a project given its title and optionally summary,
 return its identifier.
 
-The list of valid project templates is available from the template server:
-L<https://secure.gooddata.com/projectTemplates/>.
-
-Valid db engine drivers are 'Pg' (default) and 'mysql'.
-
 =cut
 
 sub create_project
@@ -341,9 +299,6 @@ sub create_project
 	my $self = shift;
 	my $title = shift or die 'No title given';
 	my $summary = shift || '';
-	my $template = shift;
-	my $driver= shift;
-	my $token = shift;
 
 	# The redirect magic does not work for POSTs and we can't really
 	# handle 401s until the API provides reason for them...
@@ -351,72 +306,15 @@ sub create_project
 
 	return $self->{agent}->post ($self->get_uri ('projects'), {
 		project => {
-			content => {
-				# No hook to override this; use web UI
-				guidedNavigation => 1,
-				($driver ? (driver => $driver) : ()),
-				($token ? (authorizationToken => $token) : ())
-			},
+			# No hook to override this; use web UI
+			content => { guidedNavigation => 1 },
 			meta => {
 				summary => $summary,
 				title => $title,
-				($template ? (projectTemplate => $template) : ()),
 			}
 	}})->{uri};
 }
 
-=item B<create_user> DOMAIN EMAIL LOGIN PASSWORD FIRST_NAME LAST_NAME PHONE COMPANY SSO_PROVIDER
-
-Create a user given its email, login, password, first name, surname, phone and optionally company,
-sso provider in domain.
-
-Returns user identifier (URI).
-
-=cut
-
-sub create_user
-{
-	my $self = shift;
-	my $domain_uri = shift || die "No domain specified";
-	my $email = shift || die "Email must be specified";
-	my $login = shift || $email;
-	my $passwd = shift;
-	my $firstname = shift;
-	my $lastname = shift;
-	my $phone = shift;
-	my $company = shift || '';
-	my $sso_provider = shift;
-
-	return $self->{agent}->post ($domain_uri."/users", { #TODO links does not exists in REST API
-		accountSetting => {
-			login => $login,
-			email => $email,
-			password => $passwd,
-			verifyPassword => $passwd,
-			firstName => $firstname,
-			lastName => $lastname,
-			phoneNumber => $phone,
-			companyName => $company,
-			($sso_provider ? (ssoProvider => $sso_provider) : ()),
-	}})->{uri};
-}
-
-=item B<get_roles> PROJECT
-
-Gets project roles.
-
-Return array of project roles.
-
-=cut
-
-sub get_roles
-{
-	my $self = shift;
-	my $project = shift;
-
-	return $self->{agent}->get (
-		$self->get_uri (new URI($project), 'roles'))->{projectRoles}{roles};
-}
 =item B<reports> PROJECT
 
 Return array of links to repoort resources on metadata server.
@@ -447,8 +345,8 @@ sub compute_report
 
 	return $self->{agent}->post (
 		$self->get_uri (qw/xtab xtab-executor3/),
-		{ report_req => { report => ''.$report }}
-	);
+		{ report_req => { report => $report }}
+	)->{reportResult2}{meta}{uri};
 }
 
 =item B<export_report> REPORT FORMAT
@@ -468,31 +366,21 @@ sub export_report
 	my $result = $self->{agent}->post (
 		$self->get_uri (qw/report-exporter exporter-executor/),
 		{ result_req => { format => $format,
-			result => $self->compute_report ($report) }}
+			report => $self->compute_report ($report) }}
 	);
 
-	# This is for new release, where location is finally set correctly;
-	$result = $result->{uri} if ref $result eq 'HASH';
-
 	# Trigger the export
 	my $exported = $self->poll (
 		sub { $self->{agent}->get ($result) },
-		sub { $_[0] and exists $_[0]->{raw} and $_[0]->{raw} ne 'null' }
+		sub { shift->{raw} ne 'null' }
 	) or die 'Timed out waiting for report to export';
 
-	# Follow the link
-	$exported = $self->{agent}->get ($exported->{uri}) if exists $exported->{uri};
-
 	# Gotten the correctly coded result?
-	my $wanted = $format;
-	my %compat = (
+	return $exported->{raw} if $exported->{type} eq {
 		png => 'image/png',
 		pdf => 'application/pdf',
 		xls => 'application/vnd.ms-excel',
-		csv => 'text/csv',
-	);
-	$wanted = $compat{$wanted} if exists $compat{$wanted};
-	return $exported->{raw} if $exported->{type} =~ /^$wanted/;
+	}->{$format};
 
 	die 'Wrong type of content returned';
 }
@@ -619,91 +507,6 @@ sub poll
         return undef;
 }
 
-=item B<create_object_with_expression> PROJECT URI TYPE TITLE SUMMARY EXPRESSION
-
-Create a new metadata object of type TYPE with EXPRESSION as the only content.
-
-=cut
-
-sub create_object_with_expression
-{
-	my $self = shift;
-	my $project = shift;
-	my $uri = shift;
-	my $type = shift or die 'No type given';
-	my $title = shift or die 'No title given';
-	my $summary = shift || '';
-	my $expression = shift or die 'No expression given';
-
-	if (defined $uri) {
-		$uri = new URI ($uri);
-	} else {
-		$uri = $self->get_uri (new URI ($project), qw/metadata obj/);
-	}
-
-	return $self->{agent}->post (
-		$uri,
-		{ $type => {
-			content => {
-				expression => $expression
-			},
-			meta => {
-				summary => $summary,
-				title => $title,
-			}
-		}}
-	)->{uri};
-}
-
-=item B<create_report_definition> PROJECT URI TITLE SUMMARY METRICS DIM FILTERS
-
-Create a new reportDefinition in metadata.
-
-=cut
-
-sub create_report_definition
-{
-	my $self = shift;
-	my $project = shift;
-	my $uri = shift;
-	my $title = shift or die 'No title given';
-	my $summary = shift || '';
-	my $metrics = shift || [];
-	my $dim = shift || [];
-	my $filters = shift || [];
-
-	if (defined $uri) {
-		$uri = new URI ($uri);
-	} else {
-		$uri = $self->get_uri (new URI ($project), qw/metadata obj/);
-	}
-
-	return $self->{agent}->post (
-		$uri,
-		{ reportDefinition => {
-			content => {
-				filters => [ map +{ expression => $_ }, @$filters ],
-				grid => {
-					columns => [ "metricGroup" ],
-					metrics => [ map +{ alias => '', uri => $_ }, @$metrics ],
-					rows => [ map +{ attribute => { alias => '', uri => $_,
-						totals => [[]] } }, @$dim ],
-					sort => {
-						columns => [],
-						rows => [],
-					},
-					columnWidths => []
-				},
-				format => "grid"
-			},
-			meta => {
-				summary => $summary,
-				title => $title,
-			}
-		}}
-	)->{uri};
-}
-
 =item B<DESTROY>
 
 Log out the session with B<logout> unless not logged in.
@@ -745,23 +548,15 @@ L<WWW::GoodData::Agent> -- GoodData API-aware user agent
 
 =head1 COPYRIGHT
 
-Copyright 2011, 2012, 2013 Lubomir Rintel
-
-Copyright 2012, 2013 Adam Stulpa, Jan Orel, Tomas Janousek
+Copyright 2011, Lubomir Rintel
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 Lubomir Rintel C<lkundrak@v3.sk>
 
-Adam Stulpa C<adam.stulpa@gooddata.com>
-
-Jan Orel C<jan.orel@gooddata.com>
-
-Tomas Janousek C<tomi@nomi.cz>
-
 =cut
 
 1;
