@@ -26,7 +26,7 @@ use strict;
 use warnings;
 
 use base qw/LWP::UserAgent/;
-use JSON;
+use JSON::XS;
 
 our $VERSION = '1.0';
 
@@ -90,26 +90,6 @@ sub post
 	return $self->SUPER::post ($uri, @args);
 }
 
-=item put URI, BODY, PARAMS
-
-Constructs and issues a PUT request.
-
-Compared to stock L<LWP::UserAgent>, the extra body parameter
-is encoded into JSON and set as request content, which is the
-only way to set the request content.
-
-The rest of parameters are passed to L<LWP::UserAgent> untouched.
-
-=cut
-
-sub put
-{
-	my ($self, $uri, $body, @args) = @_;
-	push @args,'Content-Type' => 'application/json',
-		Content => encode_json ($body);
-	return $self->SUPER::put ($uri, @args);
-}
-
 =item delete URI
 
 Convenience method for constructing and issuing a DELETE request.
@@ -147,12 +127,9 @@ sub request
 	# Pass processed response from subrequest (redirect)
 	return $response if ref $response eq 'HASH';
 
-	# Do not bother checking content and type if there's none
-	return undef if $response->code == 204;
-
 	# Decode
-	my $decoded = eval { decode_json ($response->content) }
-		if $response->header ('Content-Type') =~ /^application\/json(;.*)?/;
+	my $decoded = eval { new JSON::XS->allow_nonref->decode ($response->content) }
+		if $response->header ('Content-Type') eq 'application/json';
 	$decoded = {
 		type => $response->header ('Content-Type'),
 		raw => $response->content,
@@ -163,6 +140,7 @@ sub request
 		# Apache::Error exceptions lack error wrapper
 		$decoded = $decoded->{error} if exists $decoded->{error};
 		my $request_id = $response->header ('X-GDC-Request') || "";
+		$request_id =~ s/:.*//;
 		$request_id = " (Request ID: $request_id)" if $request_id;
 		die $response->status_line.$request_id unless exists $decoded->{message};
 		die sprintf ($decoded->{message}, @{$decoded->{parameters}}).$request_id;
@@ -189,9 +167,7 @@ L<LWP::UserAgent> -- Perl HTTP client
 
 =head1 COPYRIGHT
 
-Copyright 2011, 2012, 2013 Lubomir Rintel
-
-Copyright 2012 Jan Orel
+Copyright 2011, Lubomir Rintel
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
